# ProjectorPWS
# by Eresse <eresse@eresse.net>

# External Includes
require 'savon'
require 'aromat'

# Internal Includes
require 'projector_pws/version'

# ProjectorPWS Module:
# Root Module for ProjectorPWS.
module ProjectorPWS

	# Legacy Module:
	# Provides access to Legacy Web Services
	module Legacy

		# Service Definition
		WSDL = 'https://secure.projectorpsa.com/OpsProjectorWebSvc/OpsProjectorSvc.asmx?wsdl'

		# URL Path
		URL_PATH = 'OpsProjectorWebSvc/OpsProjectorSvc.asmx'

		# Base URL
		BASE_URL = 'https://secure.projectorpsa.com'

		# Open Client
		def self.open url = BASE_URL
			c = Savon.client soap_version: 2, wsdl: WSDL, endpoint: service_url(url), env_namespace: :soap
			return yield c if block_given?
			c
		end

		# Pass with Authenticate
		def self.with_auth username, password, account_code = nil, url = BASE_URL

			# Prepare Creds
			creds = {}
			creds['tns:AccountName'] = account_code if account_code
			creds['tns:EmailAddress'] = username
			creds['tns:Password'] = password

			# Open API
			open(url) { |c| yield c, { message: {}, soap_header: { 'tns:OpsAuthenticationHeader' => creds } } }
		end

		# Get URL for Account
		def self.get_url username, password, account_code = nil
			with_auth(username, password, account_code) { |c, params| c.call(:get_web_service_url, params).body[:get_web_service_url_response][:get_web_service_url_result][:web_service_url] }
		end

		# Export Resources
		def self.export_resources url, username, password, account_code = nil, max_rows = nil, include_inactive = false

			# Prepare Params
			rparams = {}
			rparams['tns:MaxRowsToReturn'] = max_rows if max_rows
			rparams['tns:IncludeInactiveFlag'] = include_inactive

			with_auth(username, password, account_code, url) { |c, params| c.call(:export_resources, params.merge(message: { request: { 'Parameters' => rparams } })).body[:export_resources_response][:export_resources_result][:data][:resources][:resource] }
		end

		# Export Cost Cards
		def self.export_cost_cards url, username, password, account_code = nil, min_date = nil, max_date = nil, max_rows = nil

			# Prepare Params
			rparams = {}
			rparams['tns:MinIncurredDate'] = min_date.strftime '%Y-%m-%dz' if min_date
			rparams['tns:MaxIncurredDate'] = max_date.strftime '%Y-%m-%dz' if max_date
			rparams['tns:MaxRowsToReturn'] = max_rows if max_rows

			# Make the call
			with_auth(username, password, account_code, url) { |c, params| c.call(:export_cost_cards, params.merge(message: { request: { 'Parameters' => rparams } })).body[:export_cost_cards_response][:export_cost_cards_result][:data][:cost_cards][:cost_card] }
		end

		# Export Expense Reports
		def self.export_expense_reports url, username, password, account_code = nil, numbers_after = nil, max_rows = nil

			# Prepare Params
			rparams = {}
			rparams['tns:MaxRowsToReturn'] = max_rows if max_rows
			rparams['tns:ExpenseReportNumbersAfter'] = numbers_after if numbers_after

			# Make the call
			with_auth(username, password, account_code, url) { |c, params| c.call(:export_expense_reports, params.merge(message: { request: { 'Parameters' => rparams } })).body[:export_expense_reports_response][:export_expense_reports_result][:data][:expense_reports][:expense_report] }
		end

		# Generate Service URL
		def self.service_url base_url
			"#{base_url}/#{URL_PATH}"
		end
	end
end
